function varargout = APC(signal, varargin)

% Automatic phase correction of an EPR signal
%
%   [realsignal_pc,  phi] = APC(realsignal, varargin)
%
%   signal                   - Vector
%                             original data vector
%
%
%   Harmonic                 - String 
%                              Degree of Harmonic (default is 1)
%
%
%   OUTPUT
%   realsignal_pc           - vector
%                             phase corrected data
%
%
% Algorithms for the phase correction of EPR data.
% First the imaginary part of the real valued signal
% is reconstructed using a Hilbert transform (for details see
% Hilbert_transform). Afterwards the signal is is rotated in the complex
% plane and integrated (also multiple integrations are possible) to an
% absorptive signal. The signal is evaluated regarding the optimal phase:
% Method 1: The integral of the negative parts of the absorptive signal is
% minimized.
% 
% Method 2: The sum over the first 3 and last 3 elements of the Fourier
% transform of the absorptive signal is maximized. This method is currently
% disabled.
%
% Copyright (c) 2017, Stephan Rein

%Initial phase
method = 1;
order = 1;
phi = 0;

si = size(signal);
if si(1) ~= 1
    signal = signal';
end

if isreal(signal)
    %Calculate a Hilbert transform of the real valued signal
    [realsignal, imsignal] = Hilbert_transform(signal);
    complete_sig = realsignal+1i*imsignal;
else
    complete_sig = signal;
end
%Phase sweep in the complex plane in 1 degree steps for zeroth order
min_ang = -pi;
max_ang = pi;
angles = linspace(min_ang, max_ang, 361);

%Check if additional options were passed (method or order)
if nargin > 1
    if strcmp(varargin{1}, 'Harmonic')
        order = varargin{2};
    end
end

% Field domain correction. Finding the mimimum of the absolute cumulated negative
% negative values of the integrated signal.
if method == 1
    Signal_tmp = complete_sig;
    if order > 0
        for j = 1:order
            Signal_tmp = cumtrapz(Signal_tmp);
        end
    end
    t_best = abs(trapz(Signal_tmp(Signal_tmp<0)));
    % Run loop to find the best phase
    for k = 1:length(angles)
        Signal_tmp = phase_offset(complete_sig, angles(k));
        if order > 0
            for j = 1:order
                Signal_tmp = cumtrapz(Signal_tmp);
            end
        end
        t = abs(trapz(Signal_tmp(Signal_tmp<0)));
        if t < t_best
            t_best = t;
            phi = angles(k);
        end
    end
end

% Construct the full signale
signal =  phase_offset(signal, phi);

if si(1) ~= 1
    signal = signal';
end
varargout{1} = signal;
varargout{2} = phi;

% *************************************************************************
%% Fourier space correction DISBABLED AT THE MOMENT
% *************************************************************************
% elseif method == 2
%     Signal_tmp = complete_sig;
%     if order > 0
%         for j = 1:order
%             Signal_tmp = cumsum(Signal_tmp);
%         end
%     end
%     Signal_tmp = (baseline_correction(real(Signal_tmp),1,10));
%     start = fft(Signal_tmp);
%     t_max = abs(sum(real(start(1:3)))+abs(sum(real(start(end-3:end)))));
%     for k = 1:length(angles)
%         rotate =  real(exp(1i*angles(k)).*(complete_sig));
%         Signal_tmp = rotate;
%         if order > 0
%             for j = 1:order
%                 Signal_tmp = cumsum(Signal_tmp);
%             end
%         end
%         Signal_tmp = (baseline_correction(real(Signal_tmp),1,10));
%         Signal_tmp = fft(Signal_tmp);
%         t = abs(sum(real(Signal_tmp(1:3))))+abs(sum(real(Signal_tmp(end-3:end))));
%         if t > abs(t_max)
%             t_max = abs(t);
%             phi = angles(k);
%         end
%     end
%     str = 'Fourier space method was used';
% end
% realsignal_pc =  real(exp(1i*phi)*(complete_sig));
% fprintf(str)
% fprintf('\nPhase correction was carried out with phi = %f',phi*(180/pi));
% fprintf(' degree\n');
