function [x_opt,varargout] = LevMarq_SR(fun, x0, opt)

% Levenberg-Marquardt algorithm for nonlinear optimizations
%
%
%   INPUT:
%   fun                      - Function handle
%                              Residual function which should be optimized.
%
%   x0                       - Vector
%                              Initial parameter guess.
%
%   Optional parameter opt. Passed in a structure (e.g. opt.verbose = true)
%
%   method                   - String ('diagonal')
%                              Uses diag(J'*J) instead of the unit matrix I
%                              to calculate the damping matrix.
%
%   verbose                  - Boolean
%                              If set to false the information about the
%                              optimization progress is not printed on the
%                              console. Default (true).
%
%   maxIter                  - Integer
%                              Maximum number of iterations. Default (100).
%
%   mu                       - Float
%                              Initial mu (damping factor). Needs to be
%                              positive. Default (0.2).
%
%   mu_fac                   - Integer
%                              Factor which is used to reduce/enhance mu to
%                              ensure convergence by simultaneously using
%                              the maximum stepsize. Default (2).
%
%   tol                      - Float
%                              Tolerance value for the change in the 
%                              objective function when convergence is 
%                              assumed. Default (1e-6).
%
%   tol_norm                  - Float
%                              Tolerance value for the change the norm of
%                              the "step-vector" when convergence is
%                              assumed. Default (None).
%
%   bounderies                - Vector
%                              Vector with upper and lower bounderies.
%                              Default: None.
%
%   disable_Broyden          - Boolean
%                              If set to true the Broyden's method for the
%                              approximation of the Jacobi matrix is
%                              disabled. The forward differences approach
%                              is used for each iteration step.
%                              Default (false).
%
%   nIter                    - Integer
%                              Maximum number of iterations. Alias for
%                              maxIter. Default(100).
%
%   OUTPUT:
%   x_opt                    - Vector
%                              Optimized parameters.
%
%   J (optional)             - Matrix
%                              Approximated Jacobi matrix evaluated
%                              at the found minimum.
%
%
%  Levenberg-Marquardt algorithm for nonlinear optimizations [1,2].
%  The implementation is as follows:
%
%  I. Evaluate f(x) and the Jacobian f'(x)           (forward differences)
%
%  II. Calculate the correction vector s by using either
%     s = -[f'(x)^Tf'(x)+mu^2*I]^(-1)*f'(x)^T*f(x)         (default method)
%  or
%     s = = -[f'(x)^Tf'(x)+mu^2*diag(f'(x)^Tf'(x))]^(-1)*f'(x)^T*f(x) .
%
%  III. Calculate eps:
%     eps = (||f(x)||^2-||f(x+s)||^2)/(||f(x)||^2-||f(x)+f'(x)*s||^2) .
%
%  IV. Decide if s is accepted and if mu is changed (b1 = 0.1, b2 = 0.7):
%     eps < b1 :      s is not accepted, mu = mu*mu_fac
%     b1 < eps < b2 : s is accepted, mu = mu
%     eps > b2 :      s is accepted, mu = mu/(2*mu_fac)
%
%  V. Set the new parameters to x = x+s and start again with I.
%
%  The default setting enables Broyden's method after 3 iterations to
%  approximate the Jacobi matrix. This spares valuable computation time.
%  However, Broyden's method is just a rank-one update. Therefore the
%  Jacobi matrix is recalculated every fourth iteration using forward
%  differences.
%
% [1]: K. Levenberg, Quart. Appl. Math. 2, 164-168, 1944.
% [2]: D. Marquardt, SIAM J. Appl. Math. 11, 431-441, 1963.

%**************************************************************************
% Presettings
%**************************************************************************
% Default settings (if not optionally passed)
verbose = true;
method = 1;
mu = 0.2;
mu_fac = 2;
maxIter = 100;
b1 = 0.1;
b2 = 0.7;
tol = 1e-7;
disable_Broyden = false;
boundaries = [];
tol_norm = 0;

% Check if additional options are provided
if nargin >1
    [method,verbose,disable_Broyden,mu,mu_fac,maxIter,tol,tol_norm, ...
        boundaries] =...
        checkopt(method,verbose,disable_Broyden,mu,mu_fac,maxIter,tol,opt,...
        tol_norm,boundaries);
end

% Initial Jacobi matrix and least-squares
x = x0;
s = 0;
res = feval(fun,x);
lsq = res*res';
lsqo= lsq;
J = Jacobi_appr_local(fun,x,res);
if verbose == true
    fprintf('Iteration \t   Feval\t         norm(s)\t\t    mu \n')
end

%**************************************************************************
% Levenberg-Marquardt iteration
%**************************************************************************
for i = 1:maxIter
    if verbose == true
        fprintf('%.3d\t\t%.6f\t\t%.7f\t\t%.4e', i, lsq,norm(s), mu)
        fprintf('\n')
    end
    [s,mu,lsq] = new(fun,J,x,mu,b1,b2,lsq,res,method,mu_fac,boundaries);
    % Update parameters and the Jacobi matrix
    x = x+s;
    res = feval(fun,x);
    tolerance = tol*lsq;
    if i > 1
        if tolerance > difflsq
            break
        end
        if norm(s)<tol_norm
            break
        end
    end
    difflsq = lsqo-lsq;
    lsqo = lsq;
    % Use approximation of the Jacobi matrix (Broyden's method) or forward
    % differences
    if  i > 3 && mod(i,3) ~= 0 && disable_Broyden == false
        J = J+((lsq-lsqo-J*s)*s')/(norm(s)^2);
    else
        J = Jacobi_appr_local(fun,x,res);
    end
end
x_opt = x;
varargout{1} = J;
end

%**************************************************************************
% Subfunctions
%**************************************************************************
% Function for deterimining the optimal step length in the LM-iteration
function  [s,mu,lsq] = new(fun,J,x,mu,b1,b2,lsq_old,res,method,mu_fac, ...
    boundaries)
mu_max = 1e6;
if mu > mu_max
    s = 0;
    lsq = lsq_old;
    return
end
g = J'*res';
Hess = (J'*J);
D = eye(length(x));
if method ==1
    h = (Hess+mu*D);
else
    h =(Hess+mu*diag(diag(Hess)));
    for i = 1:length(x)
        if h(i,i) == 0
            h(i,i) = 1;
        end
    end
end
s = h\(-g);
res_old = res;
lsq_old = res_old*res_old';
expected_desc = res_old*res_old'-(res_old'+J*s)'*(res_old'+J*s);
n1 = norm(s);
s = check_boundaries(x,s,boundaries);
norm_of_s= norm(s)/n1;
expected_desc = expected_desc*norm_of_s;
res = feval(fun,x+s);
lsq = res*res';
eps = (lsq_old-lsq)/(expected_desc);
if eps < b1
    mu = mu*(mu_fac);
    res = res_old;
    [s,mu,lsq] = new(fun,J,x,mu,b1,b2,lsq_old,res,method,mu_fac,...
        boundaries);
elseif eps > b2
    mu = mu/(mu_fac);
end
end

% Approximation of the Jacobi matrix using forward differences
function varargout = Jacobi_appr_local(fun,x,lsq, varargin)
paramdim = length(x);
datadim = length(lsq);
J = zeros(datadim ,paramdim );
step = 1e-6;
p = x;
% Check if additional options have been passed
if nargin >3
    step = varargin{1};
end

for  i = 1 : paramdim
    if  x(i) == 0
        p(i) = step^2;
    else
        p(i) = x(i) + step*abs(x(i));
    end
    ret = feval(fun,p);
    J(:,i) = (ret - lsq)/(p(i) - x(i));
    p(i) = x(i);
end
varargout{1} = J;
varargout{2} = J'*J;
end

function [method,verbose,disable_Broyden,mu,mu_fac,maxIter,tol, ...
    tol_norm,boundaries] = ...
    checkopt(method,verbose,disable_Broyden,mu,mu_fac,maxIter,tol,opt,...
    tol_norm,boundaries)
% Check additional input options
if isfield(opt,'method')
    if strcmpi(opt.method,'diagonal')
        method = 2;
    end
end
if isfield(opt,'verbose')
    if opt.verbose == false
        verbose = false;
    end
end
if isfield(opt,'disable_Broyden')
    if opt.verbose == true
        disable_Broyden = true;
    end
end
if isfield(opt,'mu')
    mu = abs(opt.mu);
end
if isfield(opt,'mu_fac')
    mu_fac = abs(opt.mu_fac);
end
if isfield(opt,'maxIter')
    maxIter = abs(opt.maxIter);
end
if isfield(opt,'nIter')
    maxIter = abs(opt.nIter);
end
if isfield(opt,'tol')
    tol = abs(opt.tol);
end
if isfield(opt,'boundaries')
    boundaries = opt.boundaries;
end
if isfield(opt,'tol_norm')
    tol_norm = opt.tol_norm;
end
end

function s = check_boundaries(x,s,boundaries)
% Check if boundary conditions are fulfilled
if ~isempty(boundaries)
    x_new =  x+s;
    lb = boundaries(1,:);
    ub = boundaries(2,:);
    for j = 1:length(lb)
        if x_new(j) < lb(j);
            x_new(j) = lb(j);
        end
        if x_new(j) > ub(j);
            x_new(j) = ub(j);
        end
    end
    s = -x +x_new;
end
end
