function [xaxis, signal] = LoadEPRfile(data_file, varargin)

% Load EPR files by providing either text file formats or
% Bruker file formats (EMX, ESP, ELEXSYS)
%
% INPUT:
%   data_file               - string
%                             Filename of the experimental data set
%
% OUTPUT:
%   xaxis                   - vector
%                             X-axis data vector
%
%   signal                  - vector
%                             Signal data vector
%
% Notes
% -----
% EPR data import function. The function can read text file formats,
% provided as .txt or as .dat. Two columns (field, intensity) are expected
% for a signal with only real entries. Three columns (field, realpart,
% imaginarypart) are expected for a signal which was recorded with
% quadratic dectection.
% Furthermore, Bruker file formats can be read. For this, both files, the
% binary (.DTA or .spc) and the description file (.DSC or .par) needs to be
% present. If a Bruker file should be read the filename need to be given
% without file format.
%
%
% Examples
% --------
% [B, spc] = LoadEPRfile('Test_data.txt')
%
% [B2, spc2] = LoadEPRfile('Some_EMX_file')
%
%
% Copyright (c) 2019, Stephan Rein, University of Freiburg

format = 0;
xaxis = 0;
signal = 0;
%**************************************************************************
%% READ TEXT FILE FORMAT
%**************************************************************************
try
    ending = data_file(end-3:end);
    if (strcmp(ending, '.txt') || strcmp(ending, '.dat') || ...
            strcmp(ending, '.DAT'))
        format = 1;
        data = load(data_file);
        datasize = size(data);
        if datasize(2) == 2
            signal = data(:, 2);
            xaxis = data(:, 1);
        else
            xaxis = data(:, 1);
            signal = data(:, 2)+1j*data(:, 3);
        end
    end
catch ME
    fprintf(ME.identifier)
end

%**************************************************************************
%% READ ELEXSYS FILE FORMAT
%**************************************************************************
if format ~= 1
    try
        % Open the DTA file
        fid = fopen([data_file, '.DTA'],'r','ieee-be.l64');
        
        if fid == -1
            fid = fopen(data_file,'r','ieee-be.l64');
            data_file = strtok(data_file, '.');
        end
        % Message if file reading was not sucessfull because there are
        % problems with binary
        if (fid <= 2)
            fprintf('\nProblems reading binary file "%s"',file);
            return;
        end
        signal = fread(fid,inf,'float64');
        fclose(fid);
        % Open the DSC file
        fid = fopen([data_file '.DSC'], 'r');
        % Open the DSC Textfile with all setup information
        if fid == -1
            warning('Problems reading .DSC file "%s"',file);
            return
        end
        % Read file as cell arry
        DSCfileContents = cell(0);
        idx=1;
        while true
            singleLineFromTextFile = fgetl(fid);
            if ~ischar(singleLineFromTextFile)
                break;
            end
            DSCfileContents{idx} = singleLineFromTextFile;
            idx=idx+1;
        end
        fclose(fid);
        [DESCblock, ~] = DSCparser(DSCfileContents);
        XPTS = read_keywords(DESCblock, 'XPTS');
        IMAG = read_keywords(DESCblock, 'IKKF');
        XMIN = read_keywords(DESCblock, 'XMIN');
        XWID = read_keywords(DESCblock, 'XWID');
        xaxis = linspace(XMIN, XMIN+XWID, XPTS);
        if strfind(IMAG,'CPLX')
            signal_tmp = signal(1:2:end)+1j*signal(2:2:end);
            signal = signal_tmp;
        end
        format=2;
    catch ME
        fprintf(ME.identifier)
    end
    
    %**********************************************************************
    %% READ EMX SVD-based denoisingAND ESP FILE FORMAT
    %**********************************************************************
    if format ~= 1 && format ~=2
        try
            % Open the spc binary file
            fid = fopen([data_file, '.spc'],'r','l');
            signal = fread(fid,inf,'float32');
            fclose(fid);
            str2 = strcat(data_file, '.par');
            % Read file as cell arry
            fileContents = cell(0);
            fid = fopen(str2);
            idx=1;
            while true
                singleLineFromTextFile = fgetl(fid);
                if ~ischar(singleLineFromTextFile)
                    break;
                end
                fileContents{idx} = singleLineFromTextFile;
                idx=idx+1;
            end
            fclose(fid);
            ANZ = read_keywords(fileContents, 'ANZ');
            GST = read_keywords(fileContents, 'GST');
            GSI = read_keywords(fileContents, 'GSI');
            xaxis = linspace(GST-GSI/2.0, GST+GSI/2.0, ANZ);
        catch ME
            fprintf(ME.identifier)
        end
    end
end
%**************************************************************************
%% ADDTIONAL OPTIONS
%**************************************************************************
if nargin > 1
    if strcmp(varargin{1},'imag')
        if isreal(signal)
            [realsignal, imagsignal] = Hilbert_transform(signal);
            signal = realsignal+1j*imagsignal;
        end
    else if strcmp(varargin{1},'integrate')
            signal = int_and_diff(signal, 1);
            if nargin > 2
                if strcmp(varargin{2},'integrate')
                    signal = int_and_diff(signal, 1);
                end
            end
        end
    end
end

% SUBFUNCTIONS for the .DSC input parser
    function [DESCblock, SPLblock] = DSCparser(DSCfileContents)
        % 1. Delete all lines without content
        for k=length(DSCfileContents):-1:1
            if isempty(DSCfileContents{k})
                DSCfileContents(k)=[];
            end
        end
        % 2. Delete all comments
        for k=length(DSCfileContents):-1:1
            if ~isempty(DSCfileContents{k})&& strcmp(DSCfileContents{k}(1),'*')
                DSCfileContents(k)=[];
            end
        end
        % Seperate the array in three subarrays
        DESCblock = cell(0);    % Desciption block
        SPLblock = cell(0);     % Standard parameter layer block
        Blockseparator = cell(0);
        % Blockseparator determines the position of # and saves it
        for k=1:length(DSCfileContents)
            if strcmp(DSCfileContents{k}(1),'#')
                Blockseparator{k} = k;
            end
        end
        for k=length(Blockseparator):-1:1
            if isempty(Blockseparator{k})
                Blockseparator(k) = [];
            end
        end
        % Convert the strings in the Blockseparatorvector to integers
        for k=Blockseparator{1}+1:Blockseparator{2}-1
            DESCblock{k} = DSCfileContents{k};
        end
        for k=Blockseparator{2}+1:Blockseparator{3}-1
            SPLblock{k} = DSCfileContents{k};
        end
    end

    function str_val = read_keywords(block, str)
        % Read blocks from the DSC file
        str_val = 0;
        % Find the Frequency and convert it to a float
        for k=1:length(block)
            if strfind(block{k}, str)
                str_val = block{k}(strfind(block{k},str)+...
                    4:length(block{k}));
                str_val = str2double(str_val);
                if  isnan(str_val)
                    str_val = block{k}(strfind(block{k},str)+ ...
                        4:length(block{k}));
                end
                break
            end
        end
        return;
    end
end