function varargout = SVD_denoising(spectrum, varargin)

% Denoising of an EPR signal based on singular value decompostion
%
%
%   INPUT
%   spectrum                       - Vector
%                                    Data which should be denoised.
%
%   Optional parameter
%
%   varargin{1}                    - Integer
%                                    Truncation limit for singular values.
%                                    If e.g. 20 is passed, it means that
%                                    the singular values for S(i,i)
%                                    are set to zero for S(i>20,i>20).
%
%   sigma                          - Vector
%                                    numeric, approximate standard
%                                    deviation of the signal
%
%   OUTPUT
%   vararout{1}                    - Vector
%                                    Denoised data
%
%   vararout{2}                    - Vector
%                                    Singular values of the original
%                                    (not rank-reduced) matrix.
%
%
%  A Hankel-type matrix of the Fourier transform of the signal is set up.
%  A singular value decomposition of this matrix is carried out. If an
%  additional integer is passed to the function, the threshold of
%  non-zero singular values is defined by this integer.
%  Otherwise the stop-criterion is determined automatically.
%  The stop criterion is reached when either 95% of the estimated standard
%  deviation of the noise is reached or the change of the standard
%  deviation is more than 5% of the noise within one rank-reduction step.
%  No more than 500 singular values are kept. Therefore, the maximal rank
%  of the rank-reduced Hankel matrix is 500.

% Check if the vector is transposed
dim = size(spectrum);
if dim(1) == 1
    spectrum =  spectrum';
end
edges = 30;

% Define noise estimator
standarddrev =  0.5*(std(spectrum(1:edges)-sum(spectrum(1:edges))/edges)...
    +std(spectrum(end-30:end)-sum(spectrum(end-30:end))/edges));

% Singular value decomposition of the signal in conjugated Fourier space
Y = fft(spectrum);
dimension1 = round(length(Y)/4);
dimension2 = length(Y)-dimension1;
kmat = hankel(Y(1:dimension1),Y(dimension1:end));
[U,S,V] = svd(kmat,'econ');
Sstatic = diag(S);
Sdiag = S;

% Check if threshold level is given and set up the rank-deficient matrix
if nargin > 1
    for j = length(diag(S)):-2:varargin{1}+1
        S(j,j) = 0;
        S(j-1,j-1) = 0;
    end
    Y_tmp = U*S*V';
    Y = zeros(1,length(Y));
    k = min(dimension1,dimension2);
    for i = 1:k
        vec_tmp = zeros(1,i);
        for s = 1:i
            vec_tmp(s) = Y_tmp(s,i+1-s);
        end
        Y(i) = (1/length(vec_tmp))*sum(vec_tmp);
    end
else
    if  length(diag(S)) > 500
        for j= length(diag(S)):-2:500
            S(j,j) = 0;
            S(j-1,j-1) = 0;
        end
        max_len = 500;
    else
        max_len = length(diag(S));
    end
    norm_val = 100;
    for j = max_len:-2:2
        S(j,j) = 0;
        S(j-1,j-1) = 0;
        Y_tmp = U*S*V';
        [stop, norm_val] =  reconstruct(Y_tmp, Y, standarddrev, ...
            dimension1, dimension2,spectrum,norm_val);
        if stop == true
            S(j,j) = Sdiag(j,j);
            S(j-1,j-1) = Sdiag(j-1,j-1);
            break
        end
    end
    Y_tmp = U*S*V';
    Y = zeros(1,length(Y));
    k = min(dimension1,dimension2);
    for i = 1:k
        vec_tmp = zeros(1,i);
        for s = 1:i
            vec_tmp(s) = Y_tmp(s,i+1-s);
        end
        Y(i) = (1/length(vec_tmp))*sum(vec_tmp);
    end
end

% Final inverse Fourier transform
spectrum = real(ifft(Y,'symmetric'));
varargout{1} = spectrum;
varargout{2} = Sstatic;
end

%SUBFUNCTION: Reconstruct signal (sum up the anti-diagonal elements)
% Check if auto-threshold is fulfilled
function [stop,norm_val] =  reconstruct(Y_tmp,Y,standarddrev,dimension1,...
    dimension2,spectrum,norm_old)
Y = zeros(1,length(Y));
k = min(dimension1,dimension2);
for i = 1:k
    vec_tmp = zeros(1,i);
    for s = 1:i
        vec_tmp(s) = Y_tmp(s,i+1-s);
    end
    Y(i) = (1/length(vec_tmp))*sum(vec_tmp);
end
norm_val  = std(spectrum'-real(ifft(Y, 'symmetric')));
if norm_val > 0.95*standarddrev || norm_val-norm_old > 0.05*standarddrev
    stop = true;
else
    stop = false;
end
end
