
function varargout = SpecProFi(easyspinfun,data,Sys,Vary,Exp,varargin)
%Calling the Main Program as SpecProFi
%
% GLOBAL FITTING OF EPR DATA USING EASYSSPIN SIMULATION ROUTINES
% (PEPPER, SALT, SAFFRON ....) COMBINED WITH SEMI-STOCHASTIC SAMPLING
%
%
% PARAMETERS
% ----------
%
% INPUT:
%
%   easyspinfun                    - string or cell array of strings
%                                    EasySpin function(s) which
%                                    is/are called (salt, pepper, ....)
%
%   data                           - vector or cell array of vectors.
%                                    Experimental data vector(s)
%
%   Sys                            - Sys structure of EasySpin for system
%                                    parameters
%
%   Vary                           - Vary structure of EasySpin for
%                                    parameter variations
%
%   Exp                            - Exp structure of EasySpin for
%                                    experimental parameters
%
%   varargin(1)                    - Opt structure of EasySpin for
%                                    simulation options.
%
%   varargin(2)                    - FitOpt structure of SpecProFi for
%                                    defining fitting options
%
% OUTPUT:
%   bestparameter                  - structure or cell array of structures
%                                    Set of best fit parameters
%
%   lsqbest                        - Best sum of least-squares
%
%
% NOTES
% -----
% SpecProFi uses the EasySpin simulation package for internal simulations.
% A Monte-Carlo or a Latin-Hypercube sampling is used to distribute
% initital parameters with a uniform distribution in the given ranges.
% SpecProFi is designed for the global analysis of multiple experimental
% datasets. More dtailed information can be get in the manual, available
% under https://www.radicals.uni-freiburg.de/de/software/manual .
%
% All specific fitting settings are defined in the FitOpt structure. 
% The following optimization algorithm are available:
% 
%     name                       algorithm                       toolbox
% -------------------------------------------------------------------------
% 'trust-region-reflective' | Trust-region reflective          |   Yes
% 'levenberg-marquardt'     | Levenberg-Marquardt              |   Yes
% 'LevMarq_SR'              | Levenberg-Marquardt              |   No   
% 'interior-point'          | Interior-point                   |   Yes  
% 'simplex'                 | Nelder-Mead simplex              |   No
% 'active-set'              | Active-set method                |   Yes
% 'sqp'                     | Sequential quadratic programming |   Yes
%
%
%
% Copyright (c) 2016-2019, Stephan Rein,
% University of Freiburg, stephan.rein@physchem.uni-freiburg.de


% Check the input and use defaults if necessary
if nargin > 6
    Opt = varargin{1};
    FitOpt = varargin{2};
elseif nargin == 6
     Opt.Verbosity = 0;
     FitOpt = varargin{1};
elseif nargin == 5
     Opt.Verbosity = 0;
     FitOpt.LHS = true;
     FitOpt.nLHS = 3;
end

[bestparameter,lsqbest] = SpecProFi_kernel(easyspinfun,data,Sys,Vary,Exp,Opt,FitOpt);
varargout{1} = bestparameter;
varargout{2} = lsqbest;
end
