function [derivative, data_denoised]=  Tihkonov_der_combined(xdata,data_o, level)

% Thikonov regularization of the derivative of a spectrum
%
% Usage:
%   [derivative, data_denoised]=  Tihkonov_der(data, level)
%
%
%   xdata                          - vector
%                                    original data xdata vector
%
%   data                           - vector
%                                    original data
%
%   level                          - integer
%                                    L-curve level
%
%   data_denoised                  - vector
%                                    denoised data
%
%   derivative                     - vector
%                                    derivative of the signal
%
%
% Combined regularization approach
% (c) 2017, Stephan Rein


%Building the midpoint diffential matrix
n = length(data_o); 
offdiagelements = ones(n,1);
diagelements = zeros(n,1);
DiffentialMatrix_mid = 0.5*spdiags([-offdiagelements, diagelements, offdiagelements], [-1,0,1], n, n);
DiffentialMatrix_mid(n,1) = 0.45;
DiffentialMatrix_mid(1,n) = -0.45;


%Build the inverse matrix
Integration_Matrix = inv(DiffentialMatrix_mid);
Integration_Matrix_sym = Integration_Matrix'*Integration_Matrix;


%Second approximative derivative operator
offdiagelements = ones(n,1);
diagelements = -2*offdiagelements;
DiffentialMatrix2 = spdiags([offdiagelements, diagelements, offdiagelements], [-1,0,1], n, n);
DiffentialMatrix2_sym =  DiffentialMatrix2'*DiffentialMatrix2;

%Lenght of the data vector
%Set the symmetrical right part to 0
data = (fft(data_o));
for k = round(length(data_o)/2):length(data_o)
    data(k) = 0;
end

data = data/max(abs(data));

%Building the midpoint diffential matrix
v = 0:1/(length(xdata)):1-1/(length(xdata));
length(v)
D = diag(v);
InvD = pinv(D);


%L-Curve for different regularization parameters
%Allocation
Lcurve1 = zeros(1,20);
Lcurve2 = zeros(1,20);


factor_a = 1*10e-7;
factor_b = 0.2*10e-7;
%L-Curve-LOOP
for i = 1:20
    lambda = ((i*2))^6;
    %Fourier control
    derivative_tmp = 1*(InvD*InvD'+lambda*factor_a*(eye(n)))\(InvD'*data);
    Int_tmp = InvD*derivative_tmp;
    Int_tmp(1) = data(1);
    data_tmp = real(ifft(Int_tmp));
    %Real space control
    derivative = (Integration_Matrix_sym+factor_b*lambda*DiffentialMatrix2_sym)\(Integration_Matrix'*data_tmp);
    spec_int = cumsum(derivative)/max(abs(cumsum(derivative)));
    spec_int = spec_int-1/15*sum(spec_int(1:15));
    spec_int = spec_int/max(abs(spec_int));
    
    %L-Curve-Vectors
    Lcurve1(i) = real(log10((1*norm(derivative)+norm(DiffentialMatrix2_sym*derivative))^2));
    Lcurve2(i) = real(log10(norm(real(data)-real(fft(spec_int)))^2));
end

%Final evaluation of the given L-curve value
lambda = ((level*2))^6;
%Fourier control
derivative_tmp = 1*(InvD*InvD'+lambda*factor_a*(eye(n)))\(InvD'*data);
Int_tmp = InvD*derivative_tmp;
Int_tmp(1) = data(1);
data_tmp = real(ifft(Int_tmp));
%Real space control
derivative = (Integration_Matrix_sym+factor_b*lambda*DiffentialMatrix2_sym)\(Integration_Matrix'*data_tmp);


%Create denoised output
data_denoised = cumtrapz(derivative);

figure(111)
hold on
plot(Lcurve2, Lcurve1,'.');
plot(Lcurve2(level), Lcurve1(level),'.','Color',[1 0 0]);
ylabel('log(||{\itD_2x}||+||{\itx}||)');
xlabel('log||{\itID_1x-y}||');
axis([min(Lcurve2) max(Lcurve2)   min(Lcurve1) max(Lcurve1) ])
title('L-Curve-Plot');
box on

A = [Lcurve2; Lcurve1];
fileID = fopen('L-curve_differentiation.txt','w');
fprintf(fileID,'%6.6f %12.8f\n',A);
fclose(fileID);


%IF plots are required for manual
%% set size of the figure and its orientation
set(111,'Units','centimeters','PaperSize',[1.2*7.05 7.05])
set(111,'PaperOrientation','portrait')
set(111,'PaperPosition',[0 0 get(111,'PaperSize')])

% Improve font sizes
textfontsize = 9;
small = [gca ];%#ok<NBRAK> %hLeg];%
normal = [get(gca,'XLabel') get(gca,'YLabel')];%
set(small,  'FontSize',round(.8*textfontsize), 'FontName','Arial')
set(normal, 'FontSize',round(.8*textfontsize), 'FontName','Arial')

%% save figure as pdf and png
%print (111,'-dpdf', 'L_Curve_one_Lorentzian_combined');

end