function [x_opt,varargout] = TrustRegion_SR(fun, x0, opt)

% Trust region reflective for nonlinear least-square problems
%
% Usage:
%   [x_opt,varargout] = TrustRegion_SR(fun, x0, opt)
%
%   INPUT
%   fun                     - Function handle
%                             Objective function which should be optimized
%
%   x0                      - Vector
%                             Initial parameter guess
%
%   Optional parameter opt. Passed in a structure (e.g. opt.verbose = true)
%
%
%   verbose                  - Boolean
%                              If set to false the information about the
%                              optimization progress is not printed on the
%                              console. Default (true).
%
%   maxIter                  - Integer
%                              Maximum number of iterations. Default (100)
%
%   tol                      - float
%                              Tolerance value when convergence is assumed.
%                              Default (5e-7)
%
%   nIter                    - Integer
%                              Maximum number of iterations. Alias for
%                              maxIter. Default(100)
%
%   OUTPUT
%   x_opt                   - Vector
%                             Optimized parameter
%
%   J (optional)            - Matrix
%                             Approximated Jacobi matrix at the minimum.
%
%   H (optional)            - Hessian
%                             Approximated Hessian (J^T*J).
%
% The implemented trust-region algorithm works as follow:
%
% I. Calculate f(x) and the Jacobian f'(x) (using forward differences)
% II. Caculate the correction vector s by using either:
%    s = -[f'(x)^Tf'(x)+mu^2*I]^(-1)*f'(x)^T*f(x)  (default method)
% or
%    s = = -[f'(x)^Tf'(x)+mu^2*diag(f'(x)^Tf'(x))]^(-1)*f'(x)^T*f(x)
% III. Calculate eps:
%    eps = (||f(x)||^2-||f(x+s)||^2)/(||f(x)||^2-||f(x)+f'(x)*s||^2)
% IV. Decide if s is accepted and if mu is changed (b1 = 0.4, b2 = 0.7):
%    eps < b1 :      s is not accepted, mu = mu*mu_fac
%    b1 < eps < b2 : s is accepted, mu = mu
%    eps > b2 :      s is accepted, mu = mu/(2*mu_fac)
% V. Set the new paramters to x = x+s and start again with I.
%
% The default setting enables after 3 iterations Broyden's method to
% approximate the Jacobi matrix. This spares valuable computation time.
% However, Broyden's method is just a rank-one update. Therefore the Jacobi
% matrix is recalculated every fourth iteration step using forward
% differences.
%
% Copyright (c) 2018, Stephan Rein



%Initial guess
x = x0;
%Default paramter
tol = 1e-7;
Delta_max = 1024;
Delta_min = 1e-6;
Delta_0 =0.5;
maxIter = 1;
verbose = true;
boundaries = [];
%Initial Jacobi and Hessian matrix and least-square evaluation
res = feval(fun,x);
[J,H] = Jacobi_appr(fun,x,res);
lsq = res*res';
if nargin  > 1
    [verbose,maxIter,tol,boundaries] = ...
        checkopt(verbose,maxIter,tol,boundaries,opt);
end
if verbose == true
    fprintf('Iter      Feval \t     norm(s)           D\n');
end
s = 1;
lsq_old = lsq;
i = 0;
while i <= maxIter;
    grad = J*res';
    t = true;
    %DIRECT CAUCHY POINT CALCULATION
    if (grad'*H*grad)<0
       tau = 1;
       t = false;
    else
       tau =  min(norm(grad)^3/(Delta_0*grad'*H*grad),1);
    end
    p_cauchy = -(tau*(Delta_0*grad))/norm(grad);
    
    %Gradient
    grad = J*res';
    s0 = p_cauchy*0;
    g0 = grad;
    d0 = -g0;
    e0 = 1e-4;
    k = 0;

    if t == true
        while k < 1000
            a = (g0'*g0)/(d0'*H*d0);
            s0 = s0+a*d0;
            g0 = g0+a*H*d0;
            beta = (g0'*g0)/(d0'*d0);
            d0 = -g0+beta*d0;
            if norm(s0) > Delta_0
                s0_tmp_f = s0-a*d0;
                check_tmp_f = grad'*(s0_tmp_f)+0.5*(s0_tmp_f)'*(H)*(s0_tmp_f);
                for s = 1:50
                    s0_tmp =  s0+a*(s/5)*d0;
                    h = norm(s0_tmp)/Delta_0;
                    s0_tmp = s0_tmp/h;
                    check_tmp_2 = grad'*(s0_tmp)+0.5*(s0_tmp)'*(H)*(s0_tmp);
                    if check_tmp_2<check_tmp_f
                        check_tmp_f = check_tmp_2;
                        s0_tmp_f = s0_tmp;
                    end
                end
                s0 = s0_tmp_f;
                break
            end
            if (grad'*H*grad)<=0
                s0 = s0-a*d0;
                d_sq = d0'*d0;
                sd = s0'*d0+d0'*s0;
                B_sq = -Delta_0^2+s0'*s0;
                a1 = real((-sd+sqrt((sd)^2-4*d_sq*B_sq))/(2*d_sq));
                a2 = real((-sd-sqrt((sd)^2-4*d_sq*B_sq))/(2*d_sq));
                a = max(abs(a1),abs(a2));
                s0 = s0+a*d0;
                break
            end
            
            if norm(g0) < e0
                break
            end
            k = k+1;
        end
        p_cauchy = s0;
    end
    %DIRECT CAUCHY POINT CALCULATION
    %if (grad'*H*grad)<0
    %    tau = 1;
    %else
    %    tau =  min(norm(grad)^3/(Delta_0*grad'*H*grad),1);
    %end
    %p_cauchy = -(tau*(Delta_0*grad))/norm(grad);
    %if norm(p_cauchy) > Delta_0
    %   sk = -(Delta_0*grad)/ norm(grad)^2;
    %   p_cauchy = p_cauch+sk;
    %end
    
    
    n1 = norm(p_cauchy);
    pred =  lsq_old+grad'*(p_cauchy)+0.5*(p_cauchy)'*(H)*(p_cauchy);
    expected_desc = lsq_old-pred+1e-15;
    p_cauchy = check_boundaries(x,p_cauchy,boundaries);
    norm_of_s= norm(p_cauchy)/n1;
    expected_desc = expected_desc*norm_of_s;
    
    res = feval(fun,x+p_cauchy);
    lsq = res*res';
    check = (lsq_old-lsq)/(expected_desc);
    if check < 1/4 &&  lsq_old-lsq > 0 
        Delta_0  = Delta_0/2;
    elseif check > 0.7  &&  lsq_old-lsq > 0 
            Delta_0  = 2*Delta_0;
            Delta_0  = min(Delta_0,Delta_max);
    elseif lsq_old-lsq < 0 && Delta_0 > Delta_min 
        Delta_0  = Delta_0/4;
        continue
    end
    if lsq_old-lsq < tol*1e-7 || norm(p_cauchy) < 1e-8 || Delta_0 < Delta_min  
        break
    end
    if lsq_old-lsq > 0 
        x = x+p_cauchy;
        lsq_old = lsq;
        [J,H] = Jacobi_appr(fun,x,res);
        i = i+1;
    else
        continue
    end

    fprintf('%.3i     %.8f       %.5f',s,  lsq, norm(p_cauchy));
    fprintf('      %.4f',Delta_0);
    fprintf('\n');
    s = s+1;
end
x_opt = x
varargout{1} = J;
varargout{2} = J'*J;
end


%Check additional input options
function [verbose,maxIter,tol,boundaries] = ...
    checkopt(verbose,maxIter,tol,boundaries,opt)
if isfield(opt,'verbose')
    if opt.verbose == false
        verbose = false;
    end
end
if isfield(opt,'maxIter')
    maxIter = abs(opt.maxIter);
end
if isfield(opt,'nIter')
    maxIter = abs(opt.nIter);
end
if isfield(opt,'tol')
    tol = abs(opt.tol);
end
if isfield(opt,'boundaries')
    boundaries = opt.boundaries;
end
end




function s = check_boundaries(x,s,boundaries)
%Check if boundary conditions are fulfilled
if ~isempty(boundaries)
    x_new =  x+s;
    lb = boundaries(1,:);
    ub = boundaries(2,:);
    for j = 1:length(lb)
        if x_new(j) < lb(j);
            x_new(j) = lb(j);
        end
        if x_new(j) > ub(j);
            x_new(j) = ub(j);
        end
    end
    s = -x +x_new;
end
end


