function varargout = baseline_correction(signal, varargin)

% Polynomial baseline correction for EPR data
%
% [signal, poly] = baseline_correction(signal, varargin)
%
%   signal                         - Vector
%                                    input data
%
%   varargin(1)                    - Integer
%                                    polynomial degree (default 1)
%
%   varargin(2)                    - Integer
%                                    Percentage for fit region
%
%   varargout(1)                   - Vector
%                                    baseline corrected spectrum
%
%   varargout(2)                   - Vector
%                                    polynomial baseline function
%
%
% Notes
% -----
% The function substracts a polyomial function of given degree for the
% EPR spectrum. The high-field and low-field edges are used to determine
% the polynomial function. The polynomial function is then fitted to this
% part of the EPR spectrum to determine the coefficients. The baseline
% corrected EPR signal is returned.
%
%
% Examples
% --------
% [signal_corr, ~] = baseline_correction(signal)
%
% [signal_corr, poly] = baseline_correction(signal, 3)
%
% [signal_corr] = baseline_correction(signal, 2, 15)
%
%
% Copyright (c) 2019, Stephan Rein, University of Freiburg


si = size(signal);
if si(1) ~= 1
    signal = signal';
end

%Check input and set  default average region
if ~isempty(varargin)    
    if length(varargin) == 1
          polynomial = varargin{1};
          N_average = round(length(signal)/10);
    end
    if length(varargin) ==2
          if varargin{2} > 50 || varargin{2} < 0.1
              varargin{2} = 10;
              warning('\nInvalid choice.')
              warning('\nChosen region was set to 10 percent.\n');
          end
          N_average = varargin{2}*signal/100;
          polynomial = varargin{1};
    end    
else
    N_average = round(length(signal)/10);
    polynomial = 1;
end

%Split the data vector
splitdatavec = [signal(1:N_average), signal(end-N_average+1:end)];
vec = 1:length(signal);
splitxvec = [vec(1:N_average)  vec(end-N_average+1:end)];

p = polyfit(splitxvec,splitdatavec,polynomial);
poly = polyval(p,vec);
signal_corr = signal-poly;

%Return transposed vector if wished
if si(1) ~= 1
    signal_corr = signal_corr';
end

% Output
varargout{1} = signal_corr;
varargout{2} = poly;
return 







