function varargout =  diff_Tikhonov(data, varargin)

% Tikhonov regularization for the derivative of an EPR spectrum
%
%
%   INPUT
%   data                           - Vector
%                                    Original data/spectrum
%
%   Optional parameters. Passed in a structure (e.g. Option.bound = 'off')
%
%   adjustment                     - String ('small' or 'large')
%                                    Reduces/Increases the internally 
%                                    chosen regularization parameter by a
%                                    factor of 10.
%
%   bound                          - String ('off' or 'on')
%                                    Disables boundary conditions for the
%                                    differentiation matrix. Slower
%                                    evaluation but respects non-zero
%                                    edges.
%
%   eval                           - String ('fast')
%                                    Significant reduction of calculation
%                                    time due to spline interpolation but
%                                    less accurate evaluation of the 
%                                    regularization parameter.
%
%   method                         - String ('AICC')
%                                    Use the corrected Akaike information
%                                    criterion (AICC) instead of
%                                    generalized cross validation.
%
%   returnG                        - String ('return_G')
%                                    Returns additionally the curve of the
%                                    G-function which was used for the
%                                    determination of the regularization
%                                    parameter.
%
%
%   OUTPUT
%   spectrum                       - Vector
%                                    Differentiated data/spectrum
%
%   data_denoised                  - Vector
%                                    Integrated spectrum after
%                                    regularization
%
%   G-functions (optional)         - 2 Vectors
%                                    G-function used for determination of
%                                    the regularization parameter.
%
%
%  Function for the numerical differentiation of data using Tikhonov
%  regularization with a third derivative operator as penalty expression.
%  "Generalized cross validation" or the "AICC" criterion is internally
%  used to determine the optimal regularization parameter.

% Standard values
shift = 0;
diffbound = 0;
method = 1;
maxquo = 10000;
m = 14;
step = 0.05;

% Check if additional options were passed
if nargin >1
    if isfield(varargin{1},'adjustment')
        if strcmp(varargin{1}.adjustment,'small')
            shift = -1;
        elseif strcmp(varargin{1}.adjustment,'large')
            shift = 1;
        end
    end
    if isfield(varargin{1},'bound')
        if strcmp(varargin{1}.bound ,'off')
            diffbound  = 1;
        end
    end
    if isfield(varargin{1},'eval')
        if strcmp(varargin{1}.eval ,'fast')
            data_org = data;
            data = data(1:2:end);
        end
    end
    if isfield(varargin{1},'method')
        if strcmp(varargin{1}.method ,'AICC')
            method = 2;
        end
    end
end

% Avoiding singularity of the differentiation matrix (pseudo boundaries)
if mod(length(data),2) == 1 && diffbound == 0
    diffbound = 0.2;
end
% Check if the vector is transposed
dim = size(data);
if dim(1) == 1
    data =  data';
end
n = length(data);

% Normalization to the integral value of the signal
data = data/sum((data));

% Building the midpoint discrete derivative operator (sparse matrix)
offdiagelements = ones(n,1);
diagelements = zeros(n,1);
DiffentialMatrix_mid = 1*spdiags([-offdiagelements, diagelements,...
    offdiagelements], [-1,0,1], n, n);
DiffentialMatrix_mid(n,n) = -diffbound;
DiffentialMatrix_mid(n,n) = diffbound;

% Set up the inverse matrix
Integration_Matrix = inv(DiffentialMatrix_mid);
Integration_Matrix_sym = Integration_Matrix'*Integration_Matrix;

% Set up the discrete second derivative operator (sparse matrix)
diagelements = -2*ones(n,1);
DiffentialMatrix2 = spdiags([offdiagelements, diagelements,...
    offdiagelements], [-1,0,1], n, n);
DiffentialMatrix2_sym =  ((DiffentialMatrix_mid'*DiffentialMatrix2')*...
    (DiffentialMatrix_mid*DiffentialMatrix2));

% Allocation
lambda= zeros(1,m);
G= zeros(1,m);

% Loop for discrete regularization parameter for m-values
for i = 1:m
    lambda(i) = (10^i)/maxquo;
    derivative_mat = (Integration_Matrix_sym+1*lambda(i)...
        *DiffentialMatrix2_sym)\(Integration_Matrix');
    derivative = derivative_mat*data;
    %Reintegration to evaluate the least square deviation
    spec_int = (DiffentialMatrix_mid\derivative)/sum((data));
    normv = norm((spec_int-data))^(2);
    p = length(data);
    %Trace as 2-dimensional sum over a Hadamard product
    tr = sum(sum(derivative_mat'.*Integration_Matrix));
    if method == 1
        G(i) = (maxquo*normv)/(1-tr/p)^2;
    else
        G(i) = maxquo*p*log((normv/p))+((2*p)/(p-tr-1))*maxquo*tr;
    end
end

% Interpolative refinement
xx = 1:step:m;
s = 1:m;
yy = spline(s,G,xx);
[~,ind] = min(yy);
val = xx(ind);
if val > 2 && val < m-1
    val = val+shift;
end

% Final regularization parameter
lambdaf = (10^(val))/maxquo;

% Final Tikhonov evaluation
derivative = (Integration_Matrix_sym+lambdaf*DiffentialMatrix2_sym)...
    \(Integration_Matrix'*data);

% Applying interpolation of the signal if eval was set to "fast"
if nargin >1
    if isfield(varargin{1},'eval')
        if strcmp(varargin{1}.eval ,'fast')
            derivative = spline(1:2:length(data_org),...
                derivative,1:length(data_org));
        end
    end
end

% Integration to evaluate the least-square deviation
spec_int = cumsum(derivative)/max(abs(cumsum(derivative)));
data_denoised = spec_int;

% Return the vector as transposed vector if that was the input
if dim(1) == 1
    data_denoised=  data_denoised';
    derivative = derivative';
end

% Output
varargout{1} = derivative;
varargout{2} = data_denoised;
if nargin >1
    if isfield(varargin{1},'returnG')
        if strcmp(varargin{1}.returnG,'return_G')
            varargout{3} = xx;
            varargout{4} = yy;
        end
    end
end