function varargout =  diff_Tikhonov_F(data,varargin)

% Thikonov regularization for the derivative of an EPR spectrum
%
% Usage:
%   [derivative, varargout]=  diff_Tikhonov_F(data, varargin)
%
%   INPUT
%   data                           - Vector
%                                    Original data/spetrum
%
%   Optional parameter. Passed in a structure (e.g. Option.bound = 'off')
%
%   adjustment                     - String ('small', 'very_small' 
%                                    or 'large', 'very_large')
%                                    Reduces/Increases the regularization
%                                    parameter by multiplying the estimated
%                                    noise with 0.8/0.6/1.2/1.4 instead of
%                                    1. 
%
%   eval                           - String ('fast')
%                                    Significant reduction of calculation
%                                    time due to spline interpolation.
%                                    Only suited for spectra where the
%                                    signal is significantly broader than
%                                    the noise.
%
%
%   returnG                        - String ('return_G')
%                                    Returns additionally the curve of the
%                                    G-function which were used for the
%                                    determination of the regularization
%                                    parameter.
%
%
%   OUTPUT
%   spectrum                       - Vector
%                                    Differentiated data/spetrum
%
%   spectrum_int                   - Vector
%                                    Integrated spectrum after
%                                    regularization
%
%   G-function (optional)          - Vector
%                                    G-function used for determination óf
%                                    the regularization parameter.
%                               
%
%
%  Function for the numerical differentiation of data using Tikhonov
%  regularization with a unit matrix in the conjugated Fourier space. Be
%  aware that this method is only realiable if the edges (30 points) are
%  linear (no baseline drift).
%
% (c) 2018, Stephan Rein


%Standard values
fac = 1;
maxquo = 100;
m = 14;
step= 0.05;
edges = 30;

%Check if additional options were passed
if nargin >1
    if isfield(varargin{1},'adjustment')
        if strcmp(varargin{1}.adjustment,'small')
            fac = 0.8;
        elseif strcmp(varargin{1}.adjustment,'large')
            fac = 1.2;
        elseif strcmp(varargin{1}.adjustment,'very_small')
            fac = 0.6;
        elseif strcmp(varargin{1}.adjustment,'very_large')
            fac = 1.4;            
        end
    end
    if isfield(varargin{1},'eval')
        if strcmp(varargin{1}.eval ,'fast')
            data_org = data;
            data = data(1:2:end);
        end
    end
    if isfield(varargin{1},'method')
        if strcmp(varargin{1}.method ,'AICC')
        end
    end
end



%Check if the vector is transposed
dim = size(data);
if dim(1) == 1
    data =  data';
end



%Normalization to the integral
data = data/sum((data));
standarddrev =  0.5*(std(data(1:edges)-sum(data(1:edges))/edges)+std(data(end-30:end)-sum(data(end-30:end))/edges));
xdata = 1:length(data);
n = length(data);
%Building X-axis of the Fourier transform
Fs = 1/(xdata(2)-xdata(1));
v= (0:1:length(data)-1)*Fs/length(data);
v_inv = 1j./v;
v_inv(1) = 2*v_inv(2);
Integration_Matrix = spdiags(v_inv',0, n, n);
Integration_Matrix_sym = Integration_Matrix'*Integration_Matrix;
%Regularization matrix
regularMt_sym = speye(length(data));

%Fourier Transform
data_ft = fft(data);
data_ft(round(n/2)+1:end) = 0;
%Allocation
lambda= zeros(1,m);
G= zeros(1,m);
%Loop for discrete regularization parameter
for i = 1:m
    lambda(i) = (5^i)/maxquo;
    derivative_mat = (Integration_Matrix_sym+1*lambda(i)...
        *regularMt_sym)\(Integration_Matrix');
    derivative = derivative_mat*data_ft;
    %Reintegration to evaluate the least square deviation
    spec_int =  real(ifft(Integration_Matrix*derivative,'symmetric'))/sum((data));
    normv = (norm((spec_int-data)));
    p = length(data);
    G(i) = normv/(fac*sqrt(p));
end

%Interpolative refinement
xx = 1:step:m;
s = 1:m;
yy = spline(s,G,xx);
[~,ind] = max(yy(yy<standarddrev));
val = xx(ind);

%Final regularization parameter
lambdaf = (5^(val))/maxquo;

%Final evaluation of the given L-curve value
derivative = (Integration_Matrix_sym+lambdaf*regularMt_sym)...
    \(Integration_Matrix'*data_ft);
derivative = real(ifft(derivative,'symmetric'));

if nargin >1
    if isfield(varargin{1},'eval')    
         if strcmp(varargin{1}.eval ,'fast')
          derivative = spline(1:2:length(data_org),...
              derivative,1:length(data_org));
         end
    end
end

%Reintegration to evaluate the least square deviation
spec_int = cumsum(derivative)/max(abs(cumsum(derivative)));
data_denoised = spec_int;

%Return the vector as transposed vector if that was the input
if dim(1) == 1
    data_denoised=  data_denoised';
    derivative = derivative';
end

%Output
varargout{1} = derivative;
varargout{2} = data_denoised;
if nargin >1
    if isfield(varargin{1},'returnG')
        if strcmp(varargin{1}.returnG,'return_G')
            varargout{3} = xx;
            varargout{4} = yy;
        end
    end
end