function [trend, detail] = discrete_wt(Signal, varargin)

% One level discrete wavelet decompostion on a d\texttt[trendyadic scale
%
% Usage:
%   [trend, detail] = discrete_wt(Signal, family)
%
%   INPUT:
%   Signal                         - Vector
%                                    input data
%
%   OPTIONAL:
%   family                         - Integer
%                                    defines wavelet family
%
%   OUTPUT:
%   trend                          - Vector 
%                                    trend part of the DWT 
%
%
%   detail                         - Vector
%                                    detail part of the decomposition
%
%
% Discrete wavelet transforms are carried out using convolution
% with a wavelet and scaling function (defined by family) and subsequent
% downsampling.
% Different orthogonal wavelet families are available. The interger value
% of the parameter family defines the familie. The following wavelet
% functions are available:
%
%   method               family
%     1              Daubechies D6-Wavelet 
%     2              Daubechies D10-Wavelet 
%     3              Daubechies D20-Wavelet       
%     4              Symlets 10-Wavelet
%     5              Coiflets 4-Wavelet
%     6              Haar-Wavelet
%
% (c) 2017, Stephan Rein, University of Freiburg


%Define wavelet family
if nargin > 1
    family = varargin{1};
else
    family = 1;
end

% Load wavelet parameters
wavelet_parameters;

if family == 1
    h = db6f;
    g = db6m;
elseif family == 2
    h = db10f;
    g = db10m;
elseif family == 3
    h = db20f;
    g = db20m;
elseif family == 4
    h = sym10f;
    g = sym10m;
elseif family == 5
    h = coif4f;
    g = coif4m;
elseif family == 6
    h = haarf;
    g = haarm;
else
    h = db6f;
    g = db6m;
end

h = fliplr(h);
g = fliplr(g);

%Convolution without zero padding
Conv1 = conv(Signal,h,'same');
Conv2 = conv(Signal,g,'same');

%Downsampling
trend = downsamp(Conv1,1);
detail = downsamp(Conv2,1);

return
