function signal = int_and_diff(signal, order)

% Integration or differentiaton of a signal
%
%  INPUT:
%   signal                         - Vector
%                                  Input data
%
%   order                          - Integer
%                                  Order of integration (positive) or
%                                  differentiation (negative).
%
%  OUTPUT:
%   signal                         - Vector
%                                  Integrated or differentiated signal
%
% Notes
% -----
% This function integrates or differentiates EPR signals. While the
% integratzion is a well-posed problem, the differentiation is ill-posed.
% It is therefore recommended to use a pseudo-field modulation or a
% Tikhonov method for differentiation
%
%
% Examples
% --------
% Integration
% signal = int_and_diff(signal, 1)
%
% Double Integration
% signal = int_and_diff(signal, 2)
%
% Differentiation
% signal = int_and_diff(signal, -1)
%
% Copyright (c) 2018, Stephan Rein, University of Freiburg

si = size(signal);
if si(1) == 1
    signal = signal';
end

n = length(signal);
if order == 0
    return
elseif order < 0
    %Building the midpoint diffential matrix
    offdiagelements = ones(n,1);
    diagelements = zeros(n,1);
    DiffentialMatrix_mid = spdiags([-offdiagelements, diagelements, offdiagelements], [-1,0,1], n, n);
    DiffentialMatrix_mid(n,1) = 1;
    DiffentialMatrix_mid(1,n) = -1;
    for i = 1:abs(order)
        signal = DiffentialMatrix_mid*signal;
    end
    
else
    %Using cumsum as integrator
    for i = 1:abs(order)
        signal = cumsum(signal);
    end
end
if si(1) == 1
    signal = signal';
end
end
