function signal = normalize(signal, varargin)

% Normalization function for experimental EPR signals
%   
%  INPUT:
%   signal                         - Vector
%                                  input data
%
%   varargin(1)                    - String
%                                  Normalization method ('maxabs' ,
%                                  'max', 'area' or 'area_per_point')
%
%  OUTPUT:
%   signal                        - Vector
%                                   Normalized EPR spectrum
%
%
% Notes
% -----
% Function for the normalization of EPR signals. Different methods are
% available. The standard method is the normalization to the absolut
% maximum of the signal. Alternatively a normalization to the
% (non-absolute) maximum of the signal, to the area, or to the area
% normalized to the number of data points can be carried out.
%
%
% Examples
% --------
% signal = normalize(signal)
%
% signal = normalize(signal, 'area')
%
% signal = normalize(signal, 'area_per_point')
%
%
% Copyright (c) 2019, Stephan Rein, University of Freiburg

si = size(signal);
if si(1) ~= 1
    signal = signal';
end

%Check input and set  default average region
if ~isempty(varargin)
    if length(varargin) == 1
        if strcmp(varargin{1}, 'maxabs')
            signal = signal/max(abs(signal));
        elseif strcmp(varargin{1}, 'max')
            signal = signal/max(signal);
        elseif strcmp(varargin{1}, 'area')
            signal = signal/sum(abs(signal));
        elseif strcmp(varargin{1}, 'area_per_point')
            signal = length(signal)*signal/sum(abs(signal));
        end
    else
        warning('\nTo many input variables!\n')
    end
else
    signal = signal/max(abs(signal));
end
if si(1) ~= 1
    signal = signal';
end
