function varargout = phase_offset(signal, varargin)

% Phase offset of a real-valued or complex EPR signal 
%   
%  INPUT:
%   signal                         - Vector
%                                  input data
%
%   varargin(1)                    - Vector or Integer
%                                  Imaginary part or phase angle
%
%   varargin(2)                    - Integer
%                                  Phase angle
%
%  OUTPUT:
%   varargout                      - Vector
%                                  EPR signal with phase offset
%
%
% Notes
% -----
% The function shifts the phase of an EPR signal for a given phase
% offset. The input signal can be real or complex. If the input signal is
% complex it can be provided as composed signal (real and imaginary) or the
% indivdual parts can be provided.
%
%
% Examples
% --------
% phi = pi/5
% signal = phase_offset(signal, phi)
%
% signal = phase_offset(realpart, imagpart, phi)
%
% [realpart, imagpart] = phase_offset(signal, phi)
%
%
% Copyright (c) 2019, Stephan Rein, University of Freiburg

phi = 0;
si = size(signal);
if si(1) ~= 1
    signal = signal';
end

%Check real and imaginar part were passed automatically
if nargin == 2
    phi = varargin{1};
    if isreal(signal)
    [realpart, imagpart] = Hilbert_transform(signal);
    signal = realpart+1i*imagpart;
    end
end
    if nargin > 2
    if si(1) ~= 1
        varargin{1} = varargin{1}';
    end
        signal = signal +1j*varargin{1};
        phi = varargin{2};   
    end

% Change the phase
signal = signal*exp(1i*phi);

% Separate imaginary and real part
realsignal = real(signal);
imagsignal = imag(signal);

if si(1) ~= 1
    signal = signal';
    realsignal = real(signal);
    imagsignal = imag(signal);
end

% Return the signal with phase offset
varargout{1} = signal;
if nargout > 1
    varargout{1} = realsignal;
    varargout{2} = imagsignal;
end
    
    