function varargout =  pseudomod(xdata, data, modAmp, varargin)

% Pseudo-field modulation of an absorptive EPR signal
%
% [signal, poly] = baseline_correction(signal, varargin)
%
% INPUT
%   data                           - vector
%                                    magnetic field data
%
%   xdata                          - vector
%                                    original intensity vector
%
%   modAmp                         - float
%                                    modulation amplitude in mT
%
%   varargin(1)                    - integer
%                                    Number of points at which the signal is
%                                    truncated in the conjugated Fourier
%                                    space
%
% OUTPUT:
%   varargout(1)                   - Vector
%                                    Pseudo-field modulated signal
%
%   varargout(1)                   - Vector
%                                    Denoised, integrated signal
%
% Notes
% -----
% Uses Bessel function of first kind as modulation kernel in Fourier space
% to carrie out a pseudo-field modulation. The modulation amplitude needs
% to be provided by the user.
%
%
% Examples
% --------
%
% modAmp = 0.1 
% signal_diff = pseudomod(field, signal, modAmp)
%
% [signal_diff, signal_denoise] = pseudomod(field, signal, 0.2)
%
% Cutoffpoints = 100
% signal_diff = pseudomod(field, signal, 0.1, Cutoffpoints)
%
%
% Copyright (c) 2018, Stephan Rein, University of Freiburg


si = size(data);
if si(1) == 1
    data = data';
    xdata = xdata';
end

%Create conjugated Fourier stepsize
step = xdata(2)-xdata(1);
n = length(xdata);
fator = n*step;
if mod(length(xdata), 2) == 0
    fxdata = [(0:1:n/2-1),(-n/2:1:-1)]/fator;
else
    fxdata = [(0:1:(n-1)/2-1),(-(n-1)/2:1:-1)]/fator;
end

%Build Bessel function of first kind in Fourier space
%Conversion from Gauss to mT and transform to frequency
level = modAmp*pi;
FTBessel = besselj(1,fxdata*level);
FTBessel = FTBessel';

%Transform cw-EPR in Fourier space
ftdata = fft(data);

%Additional option for truncation
if nargin > 3;
    if varargin{1} >= 1
        if varargin{1} < 30
            varargin{1} = 30;
        end       
        for i = 1:length(ftdata);
            if i > varargin{1};
                ftdata(i) =0;
            end
        end
    end
end
    
%Covolution as multiplication in Fourier space
pseudomodataift = 1j*(FTBessel).*ftdata;

%Inverse Fourier transform and integration
derivative = real(ifft(pseudomodataift,'symmetric'));
varargout{1} = derivative;
data_denoised = cumsum(derivative);
varargout{2} = data_denoised/max(data_denoised);
if si(1) == 1
  varargout{1} = varargout{1}';
  varargout{2} = varargout{2}';
end
end