function [field, Int_tmp] = simulation_kernel(easyspinfun,Sys, Exp, FitOpt, ...
    Opt, weighting,number_of_spc)

%  Kernel function, which calls EasySpin routines
%
%  Internal kernel. EasySpin simulation routines ('pepper', 'garlic',
%  'salt', 'chili', .....) are called. Different workarounds for treating
%  global paramters but different linewidths, or different isotopes, are
%  also covered by this function.

% Allocation
Sys_tmp = Sys;
Exp_tmp = Exp;
Sys_static = Sys;
Exp_static = Exp;
Int_tmp_p = cell(length(Exp),1);
field = cell(length(Exp),1);
Int_tmp = cell(length(Exp),1);
for b=1:number_of_spc
    %Division of different linewidths of a global fit
    [Sys, Exp] = separate_vary(Sys, Sys_tmp, Exp, Exp_tmp, b);
    Exp_tmp = Exp_static;
    Sys_tmp = Sys_static;
    % Define EasySpin function
    simFunction = str2func(easyspinfun{b});
    try
        % Call EasySpin as intrinsic or manual linear combination
        if length(Exp) > number_of_spc && length(Sys) == length(Exp)
            for s = 1:length(Exp)
                if isfield(FitOpt, 'diff_isotopes')
                    Sys{s}.Nucs = FitOpt.diff_isotopes{s};
                end
                %Call Easyspin
                [field_p,Int_tmp_p{s}] = simFunction(Sys{s},Exp{s},Opt);
            end
            Int_tmp{b} = Sys{1}.weight.*Int_tmp_p{1};
            for k = 2:length(Exp)
                Int_tmp{b} = Int_tmp{b} + Sys{k}.weight.*Int_tmp_p{k};
            end
            field{b} = field_p;
        else
            if length(Sys) > 1
                [Sys, ~] = check_Sysprop(Sys, FitOpt, b);
                [field{b}, Int_tmp{b}] = simFunction(Sys, Exp{b},Opt);
            else
                [field{b}, Int_tmp{b}] = isomixture(Sys, FitOpt, Exp, Opt, ...
                                                    b, simFunction);
            end
        end
    catch ME
        fprintf(ME.identifier)
        warning('Execption raised. EasySpin exploaded.\n')
    end

    %% POST-PROCESSING STEPS
    Int_tmp{b} = Int_tmp{b}/max(abs(Int_tmp{b}));
    if FitOpt.AbsScale == false
        Int_tmp{b} = Int_tmp{b}/max(abs(Int_tmp{b}));
    end
    % Multiply with the defined weighting factor
    try
        Int_tmp{b} =  Int_tmp{b}.*weighting{b};
    catch ME
        Int_tmp{b} =  Int_tmp{b}.*1;
    end
    % If more than one spin system, sum weighted and normalise
    if isfield(FitOpt,'maxAbs')
        if FitOpt.maxAbs == true
            Int_tmp{b} = Int_tmp{b}/max(abs(Int_tmp{b}));
        end
    end
end
end

%% SUBFUNCTIONS

function A = redefine_HF(A, ratio)
TrA = 1/3*sum(A);
Adipolar = A-ones(1,length(A)).*TrA;
Adipolar = ratio.*Adipolar;
A = TrA+Adipolar;
end

function [Sys, len] = check_Sysprop(Sys, FitOpt, b)
len = 1;
for i=1:length(Sys)
    if isfield(FitOpt, 'diff_isotopes')
        Sys{i}.Nucs = FitOpt.diff_isotopes{b};
    end
    if isfield(FitOpt, 'isomixtures')
        if FitOpt.isomixtures{1} == b
            len = length(FitOpt.isomixtures)-4;
        end
    end
    if isfield(Sys{i}, 'logtcorr')
        if Sys{i}.logtcorr <= -13
            Sys{i}.logtcorr = -12.99;
        end
    end
    if  isfield(Sys{i}, 'lw')
        for k=1:length(Sys{1}.lw)
            if Sys{i}.lw(k) < 0
                Sys{i}.lw(k) = 0;
            end
        end
    end
    if  isfield(Sys{i}, 'lwpp')
        for k=1:length(Sys{1}.lwpp)
            if Sys{i}.lwpp(k) < 0
                Sys{i}.lwpp(k) = 0;
            end
        end
    end
end
end


function [field, Int_tmp] = isomixture(Sys, FitOpt, Exp, Opt, b,simFunction)
Int_tmp_t = 0;
[Sys, len] = check_Sysprop(Sys, FitOpt, b);

for i=1:len
    if isfield(FitOpt, 'isomixtures')
        if FitOpt.isomixtures{1} == b
            Sys{1}.Nucs = FitOpt.isomixtures{4+i};
            if i == 1
                Sys{1}.A = redefine_HF(Sys{1}.A, ...
                    FitOpt.isomixtures{3});
            else
                Sys{1}.A =  Sys{1}.A;
            end
        end
    end
    [field, Int_tmp] = simFunction(Sys{1},Exp{b},Opt);
    if isfield(FitOpt, 'isomixtures')
        if FitOpt.isomixtures{1} == b
            fac = FitOpt.isomixtures{1+i};
            Int_tmp_t = Int_tmp_t+fac*normalise(Int_tmp);
        end
    end
end
if isfield(FitOpt, 'isomixtures')
    if FitOpt.isomixtures{b} == b
        Int_tmp = Int_tmp_t;
    end
end
end

function [Sys, Exp] = separate_vary(Sys, Sys_tmp, Exp, Exp_tmp, b)
    %Division of different linewidths of a global fit
    varparams = {'lw', 'lwpp', 'logtcorr'};
    for p = 1:length(varparams)
        for t = 1:length(Sys)
            if isfield(Sys{t},varparams{p})
                si = size(Sys_tmp{t}.(varparams{p}));
                if si(1) > 1 && length(Exp) > 1
                    Sys{t}.(varparams{p}) = Sys_tmp{t}.(varparams{p})(b,:);
                end
            end
        end
    end
    if b==1
        varparams = {'Temperature', 'mwFreq', 'ModAmp', 'mwPhase'};
        for p = 1:length(varparams)
            if isfield(Exp{b},varparams{p})
                si = size(Exp_tmp{b}.(varparams{p}));
                if si(1) > 1 && length(Exp) > 1
                    Exp{b}.(varparams{p}) = Exp_tmp{b}.(varparams{p})(b,:);
                end
            end
        end
    end
end
