function [FrC, FrS] = vec_Fresnel_cs(z)

% Usage: Evaluation of FresnelC and FresnelS integrals (Vectorized)
%
%
%   INPUT
%   z                 -  Vector
%                        FresnelC(z) and FresnelS(z) are supposed to be
%                        evaluated.
%
%
%   OUTPUT
%   FrC                 -  Vector
%                        FresnelC(z).
%
%   FrS                 -  Vector
%                        FresnelS(z).
%
%
%  The initial vector z is split.
%
%  For z <= 2:
%  Calculation of the Fresnel integrals by using a Taylor series
%  (up to order 33 or 35, respectively).
%
%  For z > 2:
%  Calculation of the Fresnel integrals by using a continued fraction
%  approach.
%
%  The calculation is carried out according to:
%  "Numerical recipes in C", Second Edition, p. 255.

stop = false;
in = find(z >= 2,1, 'first');
if isempty(in)
    in = length(z)+1;
    stop = true;
end
FrC = zeros(1,length(z));
FrS = zeros(1,length(z));
s = 0;

% FresnelC integral
for k=1:4:37
    FrC(1:in-1) = FrC(1:in-1)+(-1)^(s/2)*((pi/2)^s).*...
        (z(1:in-1).^k/(k*factorial(s)));
    s = s+2;
end
if stop == false
    FrC(in:end) = 1/2+(1./(pi.*z(in:end))).*sin((pi/2).*z(in:end).^2);
end
s = 1;

% FresnelS integral
for k=3:4:35
    FrS(1:in-1) = FrS(1:in-1)+(-1)^((s-1)/2).*((pi/2)^s)*...
        (z(1:in-1).^k/(k*factorial(s)));
    s = s+2;
end
if stop == false
    FrS(in:end) = 1/2-(1./(pi.*z(in:end))).*cos((pi/2).*z(in:end).^2);
end
return
