function denoised_signal = wavelet_denoising(Signal_in, varargin)

% Discrete multilevel wavelet decomposition with soft thresholding
% for denoising an arbitrary signal.
%
% Usage:
%   denoised_signal = wavelet_denoising(Signal_in, varargin)
%
%   Signal_in                      - vector
%                                   noisy input data
%
%   varargin(1)                    - integer
%                                   (optional) defines wavelet family
%
%
%   varargin(2)                    - integer
%                                   (optional) maximal possible
%                                    decomposition level
%
% OUTPUT
%   denoised_signal               - vector
%                                   denoised data
%
% Notes
% -----
% Discrete wavelet transforms are carried out. The trend part
% is decomposed on a dyadic scale until the maximum decomposition
% level is reached. Soft median based (on the first
% detail decomposition level) treshholding is carried out.
% The following wavelet families are implemented
%
%   number               family
%     1              Daubechies D6-Wavelet 
%     2              Daubechies D10-Wavelet 
%     3              Daubechies D20-Wavelet       
%     4              Symlets 10-Wavelet
%     5              Coiflets 4-Wavelet
%     6              Haar-Wavelet 
%
%
% Examples
% --------
% signal = wavelet_denoising(signal)
% 
% family = 3;
% signal = wavelet_denoising(signal, family)
%
% family = 2;
% maxlevel = 4;
% signal = wavelet_denoising(signal, family, maxlevel)
%
%
%(c) 2017, Stephan Rein


%Default option
peldor_signal = false;

%Default output
denoised_signal = Signal_in;

%Check input and set symmlet as default wavelet
if ~isempty(varargin)
    decomplevel = varargin{1};
    waveletfamily = varargin{2};
    if length(varargin) == 3
        if  varargin{3} == true;
            peldor_signal = true;
        end
    end
    auto = false;
else
    auto = true;
    waveletfamily  = 5;
end


if peldor_signal == true
    reversig = Signal_in(2:end);
    reversig = fliplr(reversig');
    Signal_org_in = Signal_in;
    Signal_in = [reversig'; Signal_in];
end



%Decide for the decomposition level if Auto is switched on
% and expand if its not a power of 2
if floor(log2(length(Signal_in)))~=log2(length(Signal_in))
    exp_len_tmp = floor(log2(length(Signal_in))) +1;
    x = 1:length(Signal_in);
    v =Signal_in;
    xq = linspace(min(x),max(x),2^exp_len_tmp);
    Signal_tmp = interp1(x,v,xq,'spline');
    Signal = Signal_tmp';
else
    Signal = Signal_in;
end

%Decide for the decomposition level
if auto == true
    len2log_signal =  floor(log2(length(Signal_in)));
    decomplevel = len2log_signal-6;
end

%Do transform of the j-decomposition level
if decomplevel < 1
    return
end

%return if there are wrong user settings
if auto == false
    if decomplevel >= floor(log2(length(Signal_in))) || waveletfamily < 1 || waveletfamily > 6
        warning('\nWrong level of decomposition or wavelet family!\n');
        return
    end
end

%Allocate the different decomposition levels
Sig =cell(decomplevel+1);
d = cell(decomplevel);
s = cell(decomplevel);
trend = cell(decomplevel+1);

%Do transform of the j-decomposition level as dyadic decomposition
Sig{1} = Signal;
for j = 1:decomplevel
    [s{j}, d{j}] = discrete_wt(Sig{j}, waveletfamily);
    Sig{j+1} = s{j};
end

%Make the tresholding at each decomposition level (detail 1 is always 0)
detail = d{1};
%Make the tresholding at each decomposition level (detail 1 is always 0)
thresh = median(abs(detail(1:round(length(Signal_in)/2))))*sqrt(2*log(length(Signal_in)));


%Set all elements of the first detail to zero
for k=1:length(detail)
    detail(k) = 0;
end
d{1} = detail;


%Soft thresholding for all other decomposition levels
for q = 2:decomplevel
    if peldor_signal == true;
        thresh = thresh/(q^(1/3));
    end
    detail = d{q};
    for k = 1:length(detail)
        if abs(detail(k)) < thresh
            detail(k)= 0;
        else
            detail(k) = sign(detail(k))*(abs(detail(k))-thresh);
        end
    end
    d{q} = detail;
end

%Reconstruction of the denoised signal
trend{1} = Sig{decomplevel+1};
for j = 1:decomplevel
    trend{j+1} = idiscrete_wt(trend{j},d{decomplevel-j+1}, waveletfamily);
end


denoised_signal = trend{decomplevel+1};
if peldor_signal == false
    denoised_signal = denoised_signal(1:length(Signal_in));
else
    length(denoised_signal)
    length(Signal_in)
    denoised_signal = denoised_signal(length(Signal_org_in):2*length(Signal_org_in));
end
end